import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Comment(Model):

    fields = ["id", "entity_type", "added_by", "notes", "created_at", "type", "entity_id"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("comments"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("comments",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("comments"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("comments",id,"delete"), None, env, headers)



import base64
import platform
from chargebee import APIError,PaymentError,InvalidRequestError,OperationFailedError, compat
from chargebee.main import ChargeBee
from chargebee.main import Environment
from chargebee.version import VERSION

def _basic_auth_str(username):
    return 'Basic ' + base64.b64encode(('%s:' % username).encode('latin1')).strip().decode('latin1')


def request(method, url, env, params=None, headers=None):
    if not env:
        raise Exception('No environment configured.')
    if headers is None:
        headers = {}

    url = env.api_url(url)
    if method.lower() in ('get', 'head', 'delete'):
        url = '%s?%s' % (url, compat.urlencode(params))
        payload = None
    else:
        payload = compat.urlencode(params)
        headers['Content-type'] = 'application/x-www-form-urlencoded'

    headers.update({
        'User-Agent': 'ChargeBee-Python-Client v%s' % VERSION,
        'Accept': 'application/json',
        'Authorization': _basic_auth_str(env.api_key),
        'Lang-Version': str(compat.py_major_v) + "." + str(compat.py_minor_v),
        'OS-Version': platform.platform()
    })

    meta = compat.urlparse(url)
    if ChargeBee.verify_ca_certs:
        connection = compat.VerifiedHTTPSConnection(meta.netloc)
        connection.set_cert(ca_certs=ChargeBee.ca_cert_path)
    else:
        if Environment.protocol == "https":
            connection = compat.HTTPSConnection(meta.netloc)
        else:
            connection = compat.HTTPConnection(meta.netloc)    
        
    connection.request(method.upper(), meta.path + '?' + meta.query, payload, headers)
    try:
        response = connection.getresponse()
        data = response.read()
        if compat.py_major_v >= 3:
            data = data.decode('utf-8')

        return process_response(url,data, response.status)
    finally:
        connection.close()


def process_response(url,response, http_code):
    try:
        resp_json = compat.json.loads(response)
    except Exception as ex:     
        raise Exception("Response not in JSON format. Probably not a chargebee error. \n URL is " + url + "\n Content is \n" + response)
    if http_code < 200 or http_code > 299:
        handle_api_resp_error(url,http_code, resp_json)

    return resp_json


def handle_api_resp_error(url,http_code, resp_json):
    if 'api_error_code' not in resp_json:
        raise Exception("The api_error_code is not present. Probably not a chargebee error. \n URL is " + url + "\nContent is \n " + str(resp_json))

    if 'payment' == resp_json.get('type'):
        raise PaymentError(http_code, resp_json)
    elif 'operation_failed' == resp_json.get('type'):
        raise OperationFailedError(http_code, resp_json)
    elif 'invalid_request' == resp_json.get('type'):
        raise InvalidRequestError(http_code, resp_json)
    else:
        raise APIError(http_code, resp_json)





import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Customer(Model):
    class BillingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class ReferralUrl(Model):
      fields = ["external_customer_id", "referral_sharing_url", "created_at", "updated_at", "referral_campaign_id", "referral_account_id", "referral_external_campaign_id", "referral_system"]
      pass
    class Contact(Model):
      fields = ["id", "first_name", "last_name", "email", "phone", "label", "enabled", "send_account_email", "send_billing_email"]
      pass
    class PaymentMethod(Model):
      fields = ["type", "gateway", "gateway_account_id", "status", "reference_id"]
      pass
    class Balance(Model):
      fields = ["promotional_credits", "excess_payments", "refundable_credits", "unbilled_charges", "currency_code", "balance_currency_code"]
      pass
    class Relationship(Model):
      fields = ["parent_id", "payment_owner_id", "invoice_owner_id"]
      pass
    class ParentAccountAccess(Model):
      fields = ["portal_edit_child_subscriptions", "portal_download_child_invoices", "send_subscription_emails", "send_invoice_emails", "send_payment_emails"]
      pass
    class ChildAccountAccess(Model):
      fields = ["portal_edit_subscriptions", "portal_download_invoices", "send_subscription_emails", "send_invoice_emails", "send_payment_emails"]
      pass

    fields = ["id", "first_name", "last_name", "email", "phone", "company", "vat_number", "auto_collection", \
    "offline_payment_method", "net_term_days", "vat_number_validated_time", "vat_number_status", \
    "allow_direct_debit", "is_location_valid", "created_at", "created_from_ip", "exemption_details", \
    "taxability", "entity_code", "exempt_number", "resource_version", "updated_at", "locale", "billing_date", \
    "billing_date_mode", "billing_day_of_week", "billing_day_of_week_mode", "pii_cleared", "card_status", \
    "fraud_flag", "primary_payment_source_id", "backup_payment_source_id", "billing_address", "referral_urls", \
    "contacts", "payment_method", "invoice_notes", "preferred_currency_code", "promotional_credits", \
    "unbilled_charges", "refundable_credits", "excess_payments", "balances", "meta_data", "deleted", \
    "registered_for_gst", "consolidated_invoicing", "customer_type", "business_customer_without_vat_number", \
    "client_profile_id", "relationship", "use_default_hierarchy_settings", "parent_account_access", \
    "child_account_access"]


    @staticmethod
    def create(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("customers"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id), params, env, headers)

    @staticmethod
    def update_payment_method(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"update_payment_method"), params, env, headers)

    @staticmethod
    def update_billing_info(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"update_billing_info"), params, env, headers)

    @staticmethod
    def contacts_for_customer(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"contacts"), params, env, headers)

    @staticmethod
    def assign_payment_role(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"assign_payment_role"), params, env, headers)

    @staticmethod
    def add_contact(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"add_contact"), params, env, headers)

    @staticmethod
    def update_contact(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"update_contact"), params, env, headers)

    @staticmethod
    def delete_contact(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"delete_contact"), params, env, headers)

    @staticmethod
    def add_promotional_credits(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"add_promotional_credits"), params, env, headers)

    @staticmethod
    def deduct_promotional_credits(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"deduct_promotional_credits"), params, env, headers)

    @staticmethod
    def set_promotional_credits(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"set_promotional_credits"), params, env, headers)

    @staticmethod
    def record_excess_payment(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"record_excess_payment"), params, env, headers)

    @staticmethod
    def collect_payment(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"collect_payment"), params, env, headers)

    @staticmethod
    def delete(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"delete"), params, env, headers)

    @staticmethod
    def move(params, env=None, headers=None):
        return request.send('post', request.uri_path("customers","move"), params, env, headers)

    @staticmethod
    def change_billing_date(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"change_billing_date"), params, env, headers)

    @staticmethod
    def merge(params, env=None, headers=None):
        return request.send('post', request.uri_path("customers","merge"), params, env, headers)

    @staticmethod
    def clear_personal_data(id, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"clear_personal_data"), None, env, headers)

    @staticmethod
    def relationships(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"relationships"), params, env, headers)

    @staticmethod
    def delete_relationship(id, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"delete_relationship"), None, env, headers)

    @staticmethod
    def hierarchy(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"hierarchy"), params, env, headers)

    @staticmethod
    def update_hierarchy_settings(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"update_hierarchy_settings"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError
from chargebee.main import Environment

class Event(Model):
    class Webhook(Model):
      fields = ["id", "webhook_status"]
      pass

    fields = ["id", "occurred_at", "source", "user", "webhook_status", "webhook_failure_reason", \
    "webhooks", "event_type", "api_version"]

    @property
    def content(self):
        from chargebee import Content
        return Content(self.values['content'])

    @staticmethod
    def deserialize(json_data):
        try:
            webhook_data = json.loads(json_data)
        except (TypeError, ValueError) as ex:
            raise Exception("The passed json_data is not JSON formatted . " + ex.message)
        
        api_version = webhook_data.get('api_version', None)
        env_version = Environment.API_VERSION
        if api_version != None and api_version.upper() != env_version.upper(): 
            raise Exception("API version [" + api_version.upper() + "] in response does not match "
                    + "with client library API version [" + env_version.upper() + "]")  
        return Event.construct(webhook_data)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("events"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("events",id), None, env, headers)



import os.path

from chargebee.environment import Environment


class ChargeBee(object):

    default_env = None

    verify_ca_certs = True
    ca_cert_path = os.path.join(os.path.dirname(__file__), 'ssl', 'ca-certs.crt')

    @classmethod
    def configure(cls, options):
        cls.default_env = Environment(options)



class Environment(object):

    chargebee_domain = None
    protocol= "https"
    API_VERSION = "v2"

    def __init__(self, options):
        self.api_key = options['api_key']
        self.site = options['site']

        if self.chargebee_domain is None:
            self.api_endpoint = 'https://%s.chargebee.com/api/%s' % (self.site, self.API_VERSION)
        else:
            self.api_endpoint = 'http://%s.%s/api/%s' % (self.site, self.chargebee_domain, self.API_VERSION)

    def api_url(self, url):
        return self.api_endpoint + url



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class InvoiceEstimate(Model):
    class LineItem(Model):
      fields = ["id", "subscription_id", "date_from", "date_to", "unit_amount", "quantity", "amount", "pricing_model", "is_taxed", "tax_amount", "tax_rate", "unit_amount_in_decimal", "quantity_in_decimal", "amount_in_decimal", "discount_amount", "item_level_discount_amount", "description", "entity_description", "entity_type", "tax_exempt_reason", "entity_id", "customer_id"]
      pass
    class Discount(Model):
      fields = ["amount", "description", "entity_type", "entity_id"]
      pass
    class Tax(Model):
      fields = ["name", "amount", "description"]
      pass
    class LineItemTax(Model):
      fields = ["line_item_id", "tax_name", "tax_rate", "is_partial_tax_applied", "is_non_compliance_tax", "taxable_amount", "tax_amount", "tax_juris_type", "tax_juris_name", "tax_juris_code", "tax_amount_in_local_currency", "local_currency_code"]
      pass
    class LineItemTier(Model):
      fields = ["line_item_id", "starting_unit", "ending_unit", "quantity_used", "unit_amount", "starting_unit_in_decimal", "ending_unit_in_decimal", "quantity_used_in_decimal", "unit_amount_in_decimal"]
      pass
    class LineItemDiscount(Model):
      fields = ["line_item_id", "discount_type", "coupon_id", "discount_amount"]
      pass

    fields = ["recurring", "price_type", "currency_code", "sub_total", "total", "credits_applied", \
    "amount_paid", "amount_due", "line_items", "discounts", "taxes", "line_item_taxes", "line_item_tiers", \
    "line_item_discounts", "round_off_amount", "customer_id"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Estimate(Model):

    fields = ["created_at", "subscription_estimate", "invoice_estimate", "invoice_estimates", \
    "next_invoice_estimate", "credit_note_estimates", "unbilled_charge_estimates"]


    @staticmethod
    def create_subscription(params, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","create_subscription"), params, env, headers)

    @staticmethod
    def create_sub_for_customer_estimate(id, params, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"create_subscription_estimate"), params, env, headers)

    @staticmethod
    def create_sub_item_for_customer_estimate(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"create_subscription_for_items_estimate"), params, env, headers)

    @staticmethod
    def update_subscription(params, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","update_subscription"), params, env, headers)

    @staticmethod
    def update_subscription_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","update_subscription_for_items"), params, env, headers)

    @staticmethod
    def renewal_estimate(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"renewal_estimate"), params, env, headers)

    @staticmethod
    def advance_invoice_estimate(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"advance_invoice_estimate"), params, env, headers)

    @staticmethod
    def upcoming_invoices_estimate(id, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"upcoming_invoices_estimate"), None, env, headers)

    @staticmethod
    def change_term_end(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"change_term_end_estimate"), params, env, headers)

    @staticmethod
    def cancel_subscription(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"cancel_subscription_estimate"), params, env, headers)

    @staticmethod
    def pause_subscription(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"pause_subscription_estimate"), params, env, headers)

    @staticmethod
    def resume_subscription(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"resume_subscription_estimate"), params, env, headers)

    @staticmethod
    def gift_subscription(params, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","gift_subscription"), params, env, headers)

    @staticmethod
    def create_invoice(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","create_invoice"), params, env, headers)

    @staticmethod
    def create_invoice_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("estimates","create_invoice_for_items"), params, env, headers)



class APIError(Exception):

    def __init__(self, http_code,json_obj):
        Exception.__init__(self, json_obj.get('message'))
        self.json_obj = json_obj
        self.http_status_code = http_code
        self.type = json_obj.get('type')
        self.api_error_code = json_obj.get('api_error_code')
        self.param = json_obj.get('param')

        
        self.error_code = json_obj['error_code']
        self.http_code = http_code
        self.http_body = None

class PaymentError(APIError):
    def __init__(self, http_code,json_obj):
        APIError.__init__(self, http_code,json_obj)

class InvalidRequestError(APIError):
    def __init__(self, http_code,json_obj):
        APIError.__init__(self, http_code,json_obj)

class OperationFailedError(APIError):
    def __init__(self, http_code,json_obj):
        APIError.__init__(self, http_code,json_obj)





import urllib
from chargebee import util, http_request
from chargebee.main import ChargeBee
from chargebee import compat
import json

def send_list_request(method, url, params=None, env=None, headers=None):
    serialized = {}
    for k, v in list(params.items()):
        if isinstance(v, (list)):
            v = json.dumps(v)
        serialized.update({k:v})
    return send(method,url,serialized,env,headers)

def send(method, url, params=None, env=None, headers=None):
    if params is None:
        params = {}

    env = env or ChargeBee.default_env

    ser_params = util.serialize(params)

    response = http_request.request(method, url, env, ser_params, headers)

    from chargebee.result import Result
    from chargebee.list_result import ListResult
    if 'list' in response:
        return ListResult(response['list'], response.get('next_offset', None))
    return Result(response)

def uri_path(*paths):
    url = ""
    for path in paths:
        if path == None or len(str(path).strip()) < 1 :
            raise Exception("Id is None or empty")
        if compat.py_major_v >= 3:          
            url = url + "/" +  urllib.parse.quote(str(path).strip()) 
        else:
            url =  url + "/" + urllib.quote(str(util.get_val(path)))
    return url    
       



from chargebee import compat

def serialize(value, prefix=None, idx=None):

    serialized = {}

    if isinstance(value, dict):
        for k, v in list(value.items()):
            if isinstance(v, (dict, list, tuple)):
                serialized.update(serialize(v, k))
            else:
                key = ''.join([
                    prefix or '',
                    '[%s]' % k if prefix is not None else k,
                    '[%s]' % idx if idx is not None else '',
                ])
                serialized.update({key: get_val(v)})

    elif isinstance(value, (list, tuple)):
        for i, v in enumerate(value):
            serialized.update(serialize(v, prefix, i))

    else:
        if prefix is not None and idx is not None:
            key = prefix + '[' + str(idx) +']'
            serialized.update({key: get_val(value)})
        else:
            raise TypeError("only hash or arrays are allowed as value")

    return serialized

def get_val(val):
    if val is None:
        return ''
    elif isinstance(val, bool):
        return str(val).lower()
    else:
        if compat.py_major_v < 3 and isinstance(val, unicode): 
              return val.encode("utf-8")
        else:
           return val;



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Addon(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "price", "starting_unit_in_decimal", "ending_unit_in_decimal", "price_in_decimal"]
      pass

    fields = ["id", "name", "invoice_name", "description", "pricing_model", "type", "charge_type", \
    "price", "currency_code", "period", "period_unit", "unit", "status", "archived_at", "enabled_in_portal", \
    "tax_code", "taxjar_product_code", "avalara_sale_type", "avalara_transaction_type", "avalara_service_type", \
    "sku", "accounting_code", "accounting_category1", "accounting_category2", "is_shippable", "shipping_frequency_period", \
    "shipping_frequency_period_unit", "resource_version", "updated_at", "price_in_decimal", "included_in_mrr", \
    "invoice_notes", "taxable", "tax_profile_id", "meta_data", "tiers", "show_description_in_invoices", \
    "show_description_in_quotes"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("addons"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("addons",id), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("addons"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("addons",id), None, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("addons",id,"delete"), None, env, headers)

    @staticmethod
    def copy(params, env=None, headers=None):
        return request.send('post', request.uri_path("addons","copy"), params, env, headers)

    @staticmethod
    def unarchive(id, env=None, headers=None):
        return request.send('post', request.uri_path("addons",id,"unarchive"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ItemFamily(Model):

    fields = ["id", "name", "description", "status", "resource_version", "updated_at"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("item_families"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("item_families",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("item_families"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("item_families",id), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("item_families",id,"delete"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Hierarchy(Model):

    fields = ["customer_id", "parent_id", "payment_owner_id", "invoice_owner_id", "children_ids"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Subscription(Model):
    class SubscriptionItem(Model):
      fields = ["item_price_id", "item_type", "quantity", "unit_price", "amount", "free_quantity", "trial_end", "billing_cycles", "service_period_days", "charge_on_event", "charge_once", "charge_on_option"]
      pass
    class ItemTier(Model):
      fields = ["item_price_id", "starting_unit", "ending_unit", "price"]
      pass
    class ChargedItem(Model):
      fields = ["item_price_id", "last_charged_at"]
      pass
    class Addon(Model):
      fields = ["id", "quantity", "unit_price", "amount", "trial_end", "remaining_billing_cycles", "quantity_in_decimal", "unit_price_in_decimal", "amount_in_decimal"]
      pass
    class EventBasedAddon(Model):
      fields = ["id", "quantity", "unit_price", "service_period_in_days", "on_event", "charge_once", "quantity_in_decimal", "unit_price_in_decimal"]
      pass
    class ChargedEventBasedAddon(Model):
      fields = ["id", "last_charged_at"]
      pass
    class Coupon(Model):
      fields = ["coupon_id", "apply_till", "applied_count", "coupon_code"]
      pass
    class ShippingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class ReferralInfo(Model):
      fields = ["referral_code", "coupon_code", "referrer_id", "external_reference_id", "reward_status", "referral_system", "account_id", "campaign_id", "external_campaign_id", "friend_offer_type", "referrer_reward_type", "notify_referral_system", "destination_url", "post_purchase_widget_enabled"]
      pass
    class ContractTerm(Model):
      fields = ["id", "status", "contract_start", "contract_end", "billing_cycle", "action_at_term_end", "total_contract_value", "cancellation_cutoff_period", "created_at", "subscription_id", "remaining_billing_cycles"]
      pass

    fields = ["id", "currency_code", "plan_id", "plan_quantity", "plan_unit_price", "setup_fee", \
    "billing_period", "billing_period_unit", "start_date", "trial_end", "remaining_billing_cycles", \
    "po_number", "auto_collection", "customer_id", "plan_amount", "plan_free_quantity", "status", \
    "trial_start", "current_term_start", "current_term_end", "next_billing_at", "created_at", "started_at", \
    "activated_at", "gift_id", "contract_term_billing_cycle_on_renewal", "override_relationship", \
    "pause_date", "resume_date", "cancelled_at", "cancel_reason", "affiliate_token", "created_from_ip", \
    "resource_version", "updated_at", "has_scheduled_advance_invoices", "has_scheduled_changes", \
    "payment_source_id", "plan_free_quantity_in_decimal", "plan_quantity_in_decimal", "plan_unit_price_in_decimal", \
    "plan_amount_in_decimal", "offline_payment_method", "subscription_items", "item_tiers", "charged_items", \
    "due_invoices_count", "due_since", "total_dues", "mrr", "exchange_rate", "base_currency_code", \
    "addons", "event_based_addons", "charged_event_based_addons", "coupon", "coupons", "shipping_address", \
    "referral_info", "invoice_notes", "meta_data", "metadata", "deleted", "contract_term", "cancel_reason_code", \
    "free_period", "free_period_unit"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions"), params, env, headers)

    @staticmethod
    def create_for_customer(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"subscriptions"), params, env, headers)

    @staticmethod
    def create_with_items(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"subscription_for_items"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("subscriptions"), params, env, headers)

    @staticmethod
    def subscriptions_for_customer(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"subscriptions"), params, env, headers)

    @staticmethod
    def contract_terms_for_subscription(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"contract_terms"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id), None, env, headers)

    @staticmethod
    def retrieve_with_scheduled_changes(id, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"retrieve_with_scheduled_changes"), None, env, headers)

    @staticmethod
    def remove_scheduled_changes(id, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_scheduled_changes"), None, env, headers)

    @staticmethod
    def remove_scheduled_cancellation(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_scheduled_cancellation"), params, env, headers)

    @staticmethod
    def remove_coupons(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_coupons"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id), params, env, headers)

    @staticmethod
    def update_for_items(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"update_for_items"), params, env, headers)

    @staticmethod
    def change_term_end(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"change_term_end"), params, env, headers)

    @staticmethod
    def reactivate(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"reactivate"), params, env, headers)

    @staticmethod
    def add_charge_at_term_end(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"add_charge_at_term_end"), params, env, headers)

    @staticmethod
    def charge_addon_at_term_end(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"charge_addon_at_term_end"), params, env, headers)

    @staticmethod
    def charge_future_renewals(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"charge_future_renewals"), params, env, headers)

    @staticmethod
    def edit_advance_invoice_schedule(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"edit_advance_invoice_schedule"), params, env, headers)

    @staticmethod
    def retrieve_advance_invoice_schedule(id, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"retrieve_advance_invoice_schedule"), None, env, headers)

    @staticmethod
    def remove_advance_invoice_schedule(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_advance_invoice_schedule"), params, env, headers)

    @staticmethod
    def import_subscription(params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions","import_subscription"), params, env, headers)

    @staticmethod
    def import_for_customer(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"import_subscription"), params, env, headers)

    @staticmethod
    def import_contract_term(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"import_contract_term"), params, env, headers)

    @staticmethod
    def import_for_items(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"import_for_items"), params, env, headers)

    @staticmethod
    def override_billing_profile(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"override_billing_profile"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"delete"), None, env, headers)

    @staticmethod
    def pause(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"pause"), params, env, headers)

    @staticmethod
    def cancel(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"cancel"), params, env, headers)

    @staticmethod
    def cancel_for_items(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"cancel_for_items"), params, env, headers)

    @staticmethod
    def resume(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"resume"), params, env, headers)

    @staticmethod
    def remove_scheduled_pause(id, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_scheduled_pause"), None, env, headers)

    @staticmethod
    def remove_scheduled_resumption(id, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"remove_scheduled_resumption"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Invoice(Model):
    class LineItem(Model):
      fields = ["id", "subscription_id", "date_from", "date_to", "unit_amount", "quantity", "amount", "pricing_model", "is_taxed", "tax_amount", "tax_rate", "unit_amount_in_decimal", "quantity_in_decimal", "amount_in_decimal", "discount_amount", "item_level_discount_amount", "description", "entity_description", "entity_type", "tax_exempt_reason", "entity_id", "customer_id"]
      pass
    class Discount(Model):
      fields = ["amount", "description", "entity_type", "entity_id"]
      pass
    class LineItemDiscount(Model):
      fields = ["line_item_id", "discount_type", "coupon_id", "discount_amount"]
      pass
    class Tax(Model):
      fields = ["name", "amount", "description"]
      pass
    class LineItemTax(Model):
      fields = ["line_item_id", "tax_name", "tax_rate", "is_partial_tax_applied", "is_non_compliance_tax", "taxable_amount", "tax_amount", "tax_juris_type", "tax_juris_name", "tax_juris_code", "tax_amount_in_local_currency", "local_currency_code"]
      pass
    class LineItemTier(Model):
      fields = ["line_item_id", "starting_unit", "ending_unit", "quantity_used", "unit_amount", "starting_unit_in_decimal", "ending_unit_in_decimal", "quantity_used_in_decimal", "unit_amount_in_decimal"]
      pass
    class LinkedPayment(Model):
      fields = ["txn_id", "applied_amount", "applied_at", "txn_status", "txn_date", "txn_amount"]
      pass
    class DunningAttempt(Model):
      fields = ["attempt", "transaction_id", "dunning_type", "created_at", "txn_status", "txn_amount"]
      pass
    class AppliedCredit(Model):
      fields = ["cn_id", "applied_amount", "applied_at", "cn_reason_code", "cn_create_reason_code", "cn_date", "cn_status"]
      pass
    class AdjustmentCreditNote(Model):
      fields = ["cn_id", "cn_reason_code", "cn_create_reason_code", "cn_date", "cn_total", "cn_status"]
      pass
    class IssuedCreditNote(Model):
      fields = ["cn_id", "cn_reason_code", "cn_create_reason_code", "cn_date", "cn_total", "cn_status"]
      pass
    class LinkedOrder(Model):
      fields = ["id", "document_number", "status", "order_type", "reference_id", "fulfillment_status", "batch_id", "created_at"]
      pass
    class Note(Model):
      fields = ["entity_type", "note", "entity_id"]
      pass
    class ShippingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class BillingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass

    fields = ["id", "po_number", "customer_id", "subscription_id", "recurring", "status", "vat_number", \
    "price_type", "date", "due_date", "net_term_days", "currency_code", "total", "amount_paid", \
    "amount_adjusted", "write_off_amount", "credits_applied", "amount_due", "paid_at", "dunning_status", \
    "next_retry_at", "voided_at", "resource_version", "updated_at", "sub_total", "sub_total_in_local_currency", \
    "total_in_local_currency", "local_currency_code", "tax", "first_invoice", "has_advance_charges", \
    "term_finalized", "is_gifted", "expected_payment_date", "amount_to_collect", "round_off_amount", \
    "line_items", "discounts", "line_item_discounts", "taxes", "line_item_taxes", "line_item_tiers", \
    "linked_payments", "dunning_attempts", "applied_credits", "adjustment_credit_notes", "issued_credit_notes", \
    "linked_orders", "notes", "shipping_address", "billing_address", "payment_owner", "void_reason_code", \
    "deleted"]


    @staticmethod
    def create(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices"), params, env, headers)

    @staticmethod
    def create_for_charge_items_and_charges(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices","create_for_charge_items_and_charges"), params, env, headers)

    @staticmethod
    def charge(params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices","charge"), params, env, headers)

    @staticmethod
    def charge_addon(params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices","charge_addon"), params, env, headers)

    @staticmethod
    def create_for_charge_item(params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices","create_for_charge_item"), params, env, headers)

    @staticmethod
    def stop_dunning(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"stop_dunning"), params, env, headers)

    @staticmethod
    def import_invoice(params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices","import_invoice"), params, env, headers)

    @staticmethod
    def apply_payments(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"apply_payments"), params, env, headers)

    @staticmethod
    def apply_credits(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"apply_credits"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("invoices"), params, env, headers)

    @staticmethod
    def invoices_for_customer(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"invoices"), params, env, headers)

    @staticmethod
    def invoices_for_subscription(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"invoices"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("invoices",id), None, env, headers)

    @staticmethod
    def pdf(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"pdf"), params, env, headers)

    @staticmethod
    def add_charge(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"add_charge"), params, env, headers)

    @staticmethod
    def add_addon_charge(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"add_addon_charge"), params, env, headers)

    @staticmethod
    def add_charge_item(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"add_charge_item"), params, env, headers)

    @staticmethod
    def close(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"close"), params, env, headers)

    @staticmethod
    def collect_payment(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"collect_payment"), params, env, headers)

    @staticmethod
    def record_payment(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"record_payment"), params, env, headers)

    @staticmethod
    def refund(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"refund"), params, env, headers)

    @staticmethod
    def record_refund(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"record_refund"), params, env, headers)

    @staticmethod
    def remove_payment(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"remove_payment"), params, env, headers)

    @staticmethod
    def remove_credit_note(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"remove_credit_note"), params, env, headers)

    @staticmethod
    def void_invoice(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"void"), params, env, headers)

    @staticmethod
    def write_off(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"write_off"), params, env, headers)

    @staticmethod
    def delete(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"delete"), params, env, headers)

    @staticmethod
    def update_details(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("invoices",id,"update_details"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class AdvanceInvoiceSchedule(Model):
    class FixedIntervalSchedule(Model):
      fields = ["end_schedule_on", "number_of_occurrences", "days_before_renewal", "end_date", "created_at", "terms_to_charge"]
      pass
    class SpecificDatesSchedule(Model):
      fields = ["terms_to_charge", "date", "created_at"]
      pass

    fields = ["id", "schedule_type", "fixed_interval_schedule", "specific_dates_schedule"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class DifferentialPrice(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "price"]
      pass
    class ParentPeriod(Model):
      fields = ["period_unit", "period"]
      pass

    fields = ["id", "item_price_id", "parent_item_id", "price", "status", "resource_version", \
    "updated_at", "created_at", "modified_at", "tiers", "currency_code", "parent_periods"]


    @staticmethod
    def create(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices",id,"differential_prices"), params, env, headers)

    @staticmethod
    def retrieve(id, params, env=None, headers=None):
        return request.send('get', request.uri_path("differential_prices",id), params, env, headers)

    @staticmethod
    def update(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("differential_prices",id), params, env, headers)

    @staticmethod
    def delete(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("differential_prices",id,"delete"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("differential_prices"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class PaymentIntent(Model):
    class PaymentAttempt(Model):
      fields = ["id", "status", "payment_method_type", "id_at_gateway", "error_code", "error_text", "created_at", "modified_at"]
      pass

    fields = ["id", "status", "currency_code", "amount", "gateway_account_id", "expires_at", \
    "reference_id", "payment_method_type", "success_url", "failure_url", "created_at", "modified_at", \
    "resource_version", "updated_at", "customer_id", "gateway", "active_payment_attempt"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_intents"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("payment_intents",id), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("payment_intents",id), None, env, headers)



from chargebee.compat import json


class Model(object):
    
    fields = []

    def __init__(self, values, sub_types=None, dependant_types=None):
        if sub_types is None:
            sub_types = {}
        if dependant_types is None:
            dependant_types = {}
            
        self.values = values
        self.sub_types = sub_types
        self.dependant_types = dependant_types
        for field in self.fields:
            setattr(self, field, None)

    def __str__(self):
        return json.dumps(self.values, indent=4)

    def load(self, values):
        for k, v in list(values.items()):
            set_val = None
            if isinstance(v, dict) and k in self.dependant_types:
                continue
            elif isinstance(v, dict):
                set_val = self.sub_types[k].construct(v) if k in self.sub_types else v
            elif isinstance(v, (list, tuple)):
                if k in self.sub_types:
                    set_val = [self.sub_types[k].construct(x) for x in v]
                else:
                    set_val = v
            else:
                set_val = v

            if k not in ('content',):  # Skipping models properties
                setattr(self, k, set_val)

    # Returns null for any attribute that starts with cf_ to access the custom fields.
    def __getattr__(self, name):
        if( name[0:3] == "cf_"): 
            return None
        raise AttributeError("Attribute %s not found " % name) 

    @classmethod
    def construct(cls, values, sub_types=None, dependant_types=None):
        obj = cls(values, sub_types, dependant_types)
        obj.load(values)
        return obj
    
    def init_dependant(self, obj, type, sub_types={}):
        if obj.get(type) != None:
            if isinstance(obj, dict) and type in self.dependant_types:
                dependant_obj = self.dependant_types[type].construct(obj[type], sub_types)
                setattr(self, type, dependant_obj)

    def init_dependant_list(self, obj, type, sub_types={}):
        if obj.get(type) != None:
            if isinstance(obj[type],(list, tuple)) and type in self.dependant_types:
                if(self.dependant_types != None):
                    set_val = [self.dependant_types[type].construct(dt, sub_types) for dt in obj[type]]
                    setattr(self, type, set_val)

                    


import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError
from chargebee import OperationFailedError
class TimeMachine(Model):

    fields = ["name", "time_travel_status", "genesis_time", "destination_time", "failure_code", \
    "failure_reason", "error_json"]
    def wait_for_time_travel_completion(self):
        return wait_for_time_travel_completion()

    def wait_for_time_travel_completion(self, env = None):
        import time
        count = 0
        sleep_time_millis = (3000 if env == None else env.time_travel_sleep_millis)/1000.0
        
        while self.time_travel_status == 'in_progress':
            if count > 30:
                raise RuntimeError('Time travel is taking too much time')
            count+=1
            time.sleep(sleep_time_millis);
            self.values = TimeMachine.retrieve(self.name, env).time_machine.values
            self.load(self.values)
        
        if self.time_travel_status == 'failed':
            err = json.loads(self.error_json)
            raise OperationFailedError(err['http_code'], err)
        
        if self.time_travel_status in ('not_enabled', '_unknown'):
            raise RuntimeError('Time travel is in wrong state \'' + self.time_travel_status + '\'')
        
        return self


    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("time_machines",id), None, env, headers)

    @staticmethod
    def start_afresh(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("time_machines",id,"start_afresh"), params, env, headers)

    @staticmethod
    def travel_forward(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("time_machines",id,"travel_forward"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class AttachedItem(Model):

    fields = ["id", "parent_item_id", "item_id", "type", "status", "quantity", "billing_cycles", \
    "charge_on_event", "charge_once", "created_at", "resource_version", "updated_at"]


    @staticmethod
    def create(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("items",id,"attached_items"), params, env, headers)

    @staticmethod
    def update(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("attached_items",id), params, env, headers)

    @staticmethod
    def retrieve(id, params, env=None, headers=None):
        return request.send('get', request.uri_path("attached_items",id), params, env, headers)

    @staticmethod
    def delete(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("attached_items",id,"delete"), params, env, headers)

    @staticmethod
    def list(id, params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("items",id,"attached_items"), params, env, headers)

    @staticmethod
    def list_internal(params=None, env=None, headers=None):
        return request.send('get', request.uri_path("attached_items","list_internal"), params, env, headers)



from chargebee.result import Result


class ListResult(list):
    
    def __init__(self, response, next_offset):
        self.response = response
        self.next_offset = next_offset
        self._init_items()

    def _init_items(self):
        for item in self.response:
            self.append(Result(item))



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class CouponSet(Model):

    fields = ["id", "coupon_id", "name", "total_count", "redeemed_count", "archived_count", \
    "meta_data"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_sets"), params, env, headers)

    @staticmethod
    def add_coupon_codes(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_sets",id,"add_coupon_codes"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("coupon_sets"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("coupon_sets",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_sets",id,"update"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_sets",id,"delete"), None, env, headers)

    @staticmethod
    def delete_unused_coupon_codes(id, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_sets",id,"delete_unused_coupon_codes"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ThirdPartyPaymentMethod(Model):

    fields = ["type", "gateway", "gateway_account_id", "reference_id"]




from chargebee.compat import json
from chargebee.models import *


class Result(object):

    def __init__(self, response):
        self._response = response
        self._response_obj = {}

    @property
    def subscription(self):
        subscription = self._get('subscription', Subscription,
        {'subscription_items' : Subscription.SubscriptionItem, 'item_tiers' : Subscription.ItemTier, 'charged_items' : Subscription.ChargedItem, 'addons' : Subscription.Addon, 'event_based_addons' : Subscription.EventBasedAddon, 'charged_event_based_addons' : Subscription.ChargedEventBasedAddon, 'coupons' : Subscription.Coupon, 'shipping_address' : Subscription.ShippingAddress, 'referral_info' : Subscription.ReferralInfo, 'contract_term' : Subscription.ContractTerm});
        return subscription;

    @property
    def contract_term(self):
        contract_term = self._get('contract_term', ContractTerm);
        return contract_term;

    @property
    def advance_invoice_schedule(self):
        advance_invoice_schedule = self._get('advance_invoice_schedule', AdvanceInvoiceSchedule,
        {'fixed_interval_schedule' : AdvanceInvoiceSchedule.FixedIntervalSchedule, 'specific_dates_schedule' : AdvanceInvoiceSchedule.SpecificDatesSchedule});
        return advance_invoice_schedule;

    @property
    def customer(self):
        customer = self._get('customer', Customer,
        {'billing_address' : Customer.BillingAddress, 'referral_urls' : Customer.ReferralUrl, 'contacts' : Customer.Contact, 'payment_method' : Customer.PaymentMethod, 'balances' : Customer.Balance, 'relationship' : Customer.Relationship, 'parent_account_access' : Customer.ParentAccountAccess, 'child_account_access' : Customer.ChildAccountAccess});
        return customer;

    @property
    def hierarchy(self):
        hierarchy = self._get('hierarchy', Hierarchy);
        return hierarchy;

    @property
    def contact(self):
        contact = self._get('contact', Contact);
        return contact;

    @property
    def token(self):
        token = self._get('token', Token);
        return token;

    @property
    def payment_source(self):
        payment_source = self._get('payment_source', PaymentSource,
        {'card' : PaymentSource.Card, 'bank_account' : PaymentSource.BankAccount, 'amazon_payment' : PaymentSource.AmazonPayment, 'paypal' : PaymentSource.Paypal});
        return payment_source;

    @property
    def third_party_payment_method(self):
        third_party_payment_method = self._get('third_party_payment_method', ThirdPartyPaymentMethod);
        return third_party_payment_method;

    @property
    def virtual_bank_account(self):
        virtual_bank_account = self._get('virtual_bank_account', VirtualBankAccount);
        return virtual_bank_account;

    @property
    def card(self):
        card = self._get('card', Card);
        return card;

    @property
    def promotional_credit(self):
        promotional_credit = self._get('promotional_credit', PromotionalCredit);
        return promotional_credit;

    @property
    def invoice(self):
        invoice = self._get('invoice', Invoice,
        {'line_items' : Invoice.LineItem, 'discounts' : Invoice.Discount, 'line_item_discounts' : Invoice.LineItemDiscount, 'taxes' : Invoice.Tax, 'line_item_taxes' : Invoice.LineItemTax, 'line_item_tiers' : Invoice.LineItemTier, 'linked_payments' : Invoice.LinkedPayment, 'dunning_attempts' : Invoice.DunningAttempt, 'applied_credits' : Invoice.AppliedCredit, 'adjustment_credit_notes' : Invoice.AdjustmentCreditNote, 'issued_credit_notes' : Invoice.IssuedCreditNote, 'linked_orders' : Invoice.LinkedOrder, 'notes' : Invoice.Note, 'shipping_address' : Invoice.ShippingAddress, 'billing_address' : Invoice.BillingAddress});
        return invoice;

    @property
    def credit_note(self):
        credit_note = self._get('credit_note', CreditNote,
        {'line_items' : CreditNote.LineItem, 'discounts' : CreditNote.Discount, 'line_item_discounts' : CreditNote.LineItemDiscount, 'line_item_tiers' : CreditNote.LineItemTier, 'taxes' : CreditNote.Tax, 'line_item_taxes' : CreditNote.LineItemTax, 'linked_refunds' : CreditNote.LinkedRefund, 'allocations' : CreditNote.Allocation});
        return credit_note;

    @property
    def unbilled_charge(self):
        unbilled_charge = self._get('unbilled_charge', UnbilledCharge,
        {'tiers' : UnbilledCharge.Tier});
        return unbilled_charge;

    @property
    def order(self):
        order = self._get('order', Order,
        {'order_line_items' : Order.OrderLineItem, 'shipping_address' : Order.ShippingAddress, 'billing_address' : Order.BillingAddress, 'line_item_taxes' : Order.LineItemTax, 'line_item_discounts' : Order.LineItemDiscount, 'linked_credit_notes' : Order.LinkedCreditNote});
        return order;

    @property
    def gift(self):
        gift = self._get('gift', Gift,
        {'gifter' : Gift.Gifter, 'gift_receiver' : Gift.GiftReceiver, 'gift_timelines' : Gift.GiftTimeline});
        return gift;

    @property
    def transaction(self):
        transaction = self._get('transaction', Transaction,
        {'linked_invoices' : Transaction.LinkedInvoice, 'linked_credit_notes' : Transaction.LinkedCreditNote, 'linked_refunds' : Transaction.LinkedRefund, 'linked_payments' : Transaction.LinkedPayment});
        return transaction;

    @property
    def hosted_page(self):
        hosted_page = self._get('hosted_page', HostedPage);
        return hosted_page;

    @property
    def estimate(self):
        estimate = self._get('estimate', Estimate, {},
        {'subscription_estimate' : SubscriptionEstimate, 'invoice_estimate' : InvoiceEstimate, 'invoice_estimates' : InvoiceEstimate, 'next_invoice_estimate' : InvoiceEstimate, 'credit_note_estimates' : CreditNoteEstimate, 'unbilled_charge_estimates' : UnbilledCharge});
        estimate.init_dependant(self._response['estimate'], 'subscription_estimate',
        {'shipping_address' : SubscriptionEstimate.ShippingAddress, 'contract_term' : SubscriptionEstimate.ContractTerm});
        estimate.init_dependant(self._response['estimate'], 'invoice_estimate',
        {'line_items' : InvoiceEstimate.LineItem, 'discounts' : InvoiceEstimate.Discount, 'taxes' : InvoiceEstimate.Tax, 'line_item_taxes' : InvoiceEstimate.LineItemTax, 'line_item_tiers' : InvoiceEstimate.LineItemTier, 'line_item_discounts' : InvoiceEstimate.LineItemDiscount});
        estimate.init_dependant(self._response['estimate'], 'next_invoice_estimate',
        {'line_items' : InvoiceEstimate.LineItem, 'discounts' : InvoiceEstimate.Discount, 'taxes' : InvoiceEstimate.Tax, 'line_item_taxes' : InvoiceEstimate.LineItemTax, 'line_item_tiers' : InvoiceEstimate.LineItemTier, 'line_item_discounts' : InvoiceEstimate.LineItemDiscount});
        estimate.init_dependant_list(self._response['estimate'], 'invoice_estimates',
        {'line_items' : InvoiceEstimate.LineItem, 'discounts' : InvoiceEstimate.Discount, 'taxes' : InvoiceEstimate.Tax, 'line_item_taxes' : InvoiceEstimate.LineItemTax, 'line_item_tiers' : InvoiceEstimate.LineItemTier, 'line_item_discounts' : InvoiceEstimate.LineItemDiscount});
        estimate.init_dependant_list(self._response['estimate'], 'credit_note_estimates',
        {'line_items' : CreditNoteEstimate.LineItem, 'discounts' : CreditNoteEstimate.Discount, 'taxes' : CreditNoteEstimate.Tax, 'line_item_taxes' : CreditNoteEstimate.LineItemTax, 'line_item_discounts' : CreditNoteEstimate.LineItemDiscount, 'line_item_tiers' : CreditNoteEstimate.LineItemTier});
        estimate.init_dependant_list(self._response['estimate'], 'unbilled_charge_estimates',
        {'tiers' : UnbilledCharge.Tier});
        return estimate;

    @property
    def quote(self):
        quote = self._get('quote', Quote,
        {'line_items' : Quote.LineItem, 'discounts' : Quote.Discount, 'line_item_discounts' : Quote.LineItemDiscount, 'taxes' : Quote.Tax, 'line_item_taxes' : Quote.LineItemTax, 'shipping_address' : Quote.ShippingAddress, 'billing_address' : Quote.BillingAddress});
        return quote;

    @property
    def quoted_subscription(self):
        quoted_subscription = self._get('quoted_subscription', QuotedSubscription,
        {'addons' : QuotedSubscription.Addon, 'event_based_addons' : QuotedSubscription.EventBasedAddon, 'coupons' : QuotedSubscription.Coupon, 'subscription_items' : QuotedSubscription.SubscriptionItem, 'item_tiers' : QuotedSubscription.ItemTier});
        return quoted_subscription;

    @property
    def quote_line_group(self):
        quote_line_group = self._get('quote_line_group', QuoteLineGroup,
        {'line_items' : QuoteLineGroup.LineItem, 'discounts' : QuoteLineGroup.Discount, 'line_item_discounts' : QuoteLineGroup.LineItemDiscount, 'taxes' : QuoteLineGroup.Tax, 'line_item_taxes' : QuoteLineGroup.LineItemTax});
        return quote_line_group;

    @property
    def plan(self):
        plan = self._get('plan', Plan,
        {'tiers' : Plan.Tier, 'applicable_addons' : Plan.ApplicableAddon, 'attached_addons' : Plan.AttachedAddon, 'event_based_addons' : Plan.EventBasedAddon});
        return plan;

    @property
    def addon(self):
        addon = self._get('addon', Addon,
        {'tiers' : Addon.Tier});
        return addon;

    @property
    def coupon(self):
        coupon = self._get('coupon', Coupon,
        {'item_constraints' : Coupon.ItemConstraint, 'item_constraint_criteria' : Coupon.ItemConstraintCriteria});
        return coupon;

    @property
    def coupon_set(self):
        coupon_set = self._get('coupon_set', CouponSet);
        return coupon_set;

    @property
    def coupon_code(self):
        coupon_code = self._get('coupon_code', CouponCode);
        return coupon_code;

    @property
    def address(self):
        address = self._get('address', Address);
        return address;

    @property
    def event(self):
        event = self._get('event', Event,
        {'webhooks' : Event.Webhook});
        return event;

    @property
    def comment(self):
        comment = self._get('comment', Comment);
        return comment;

    @property
    def download(self):
        download = self._get('download', Download);
        return download;

    @property
    def portal_session(self):
        portal_session = self._get('portal_session', PortalSession,
        {'linked_customers' : PortalSession.LinkedCustomer});
        return portal_session;

    @property
    def site_migration_detail(self):
        site_migration_detail = self._get('site_migration_detail', SiteMigrationDetail);
        return site_migration_detail;

    @property
    def resource_migration(self):
        resource_migration = self._get('resource_migration', ResourceMigration);
        return resource_migration;

    @property
    def time_machine(self):
        time_machine = self._get('time_machine', TimeMachine);
        return time_machine;

    @property
    def export(self):
        export = self._get('export', Export,
        {'download' : Export.Download});
        return export;

    @property
    def payment_intent(self):
        payment_intent = self._get('payment_intent', PaymentIntent,
        {'payment_attempt' : PaymentIntent.PaymentAttempt});
        return payment_intent;


    @property
    def item_family(self):
        item_family = self._get('item_family', ItemFamily);
        return item_family;

    @property
    def item(self):
        item = self._get('item', Item,
        {'applicable_items' : Item.ApplicableItem});
        return item;

    @property
    def item_price(self):
        item_price = self._get('item_price', ItemPrice,
        {'tiers' : ItemPrice.Tier, 'tax_detail' : ItemPrice.TaxDetail, 'accounting_detail' : ItemPrice.AccountingDetail});
        return item_price;

    @property
    def attached_item(self):
        attached_item = self._get('attached_item', AttachedItem);
        return attached_item;

    @property
    def differential_price(self):
        differential_price = self._get('differential_price', DifferentialPrice,
        {'tiers' : DifferentialPrice.Tier, 'parent_periods' : DifferentialPrice.ParentPeriod});
        return differential_price;

    @property
    def unbilled_charges(self):
        unbilled_charges = self._get_list('unbilled_charges', UnbilledCharge,
        {'tiers' : UnbilledCharge.Tier});
        return unbilled_charges;

    @property
    def credit_notes(self):
        credit_notes = self._get_list('credit_notes', CreditNote,
        {'line_items' : CreditNote.LineItem, 'discounts' : CreditNote.Discount, 'line_item_discounts' : CreditNote.LineItemDiscount, 'line_item_tiers' : CreditNote.LineItemTier, 'taxes' : CreditNote.Tax, 'line_item_taxes' : CreditNote.LineItemTax, 'linked_refunds' : CreditNote.LinkedRefund, 'allocations' : CreditNote.Allocation});
        return credit_notes;

    @property
    def advance_invoice_schedules(self):
        advance_invoice_schedules = self._get_list('advance_invoice_schedules', AdvanceInvoiceSchedule,
        {'fixed_interval_schedule' : AdvanceInvoiceSchedule.FixedIntervalSchedule, 'specific_dates_schedule' : AdvanceInvoiceSchedule.SpecificDatesSchedule});
        return advance_invoice_schedules;

    @property
    def hierarchies(self):
        hierarchies = self._get_list('hierarchies', Hierarchy,
        {});
        return hierarchies;

    @property
    def invoices(self):
        invoices = self._get_list('invoices', Invoice,
        {'line_items' : Invoice.LineItem, 'discounts' : Invoice.Discount, 'line_item_discounts' : Invoice.LineItemDiscount, 'taxes' : Invoice.Tax, 'line_item_taxes' : Invoice.LineItemTax, 'line_item_tiers' : Invoice.LineItemTier, 'linked_payments' : Invoice.LinkedPayment, 'dunning_attempts' : Invoice.DunningAttempt, 'applied_credits' : Invoice.AppliedCredit, 'adjustment_credit_notes' : Invoice.AdjustmentCreditNote, 'issued_credit_notes' : Invoice.IssuedCreditNote, 'linked_orders' : Invoice.LinkedOrder, 'notes' : Invoice.Note, 'shipping_address' : Invoice.ShippingAddress, 'billing_address' : Invoice.BillingAddress});
        return invoices;

    @property
    def differential_prices(self):
        differential_prices = self._get_list('differential_prices', DifferentialPrice,
        {'tiers' : DifferentialPrice.Tier, 'parent_periods' : DifferentialPrice.ParentPeriod});
        return differential_prices;


    def _get_list(self, type, cls, sub_types={}, dependant_types={}, dependant_sub_types={}):
        if not type in self._response:
            return None
        
        set_val = []
        for obj in self._response[type]:
            if isinstance(obj, dict):
                model = cls.construct(obj, sub_types, dependant_types)
                for k in dependant_sub_types:
                    model.init_dependant(obj, k, dependant_sub_types[k])
                set_val.append(model)

        self._response_obj[type] = set_val
        return self._response_obj[type]

    def _get(self, type, cls, sub_types=None, dependant_types=None):
        if not type in self._response:
            return None

        if not type in self._response_obj:
            self._response_obj[type] = cls.construct(self._response[type], sub_types, dependant_types)

        return self._response_obj[type]

    def __str__(self):
        return json.dumps(self._response, indent=4)


class Content(Result):
    pass



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class VirtualBankAccount(Model):

    fields = ["id", "customer_id", "email", "scheme", "bank_name", "account_number", "routing_number", \
    "swift_code", "gateway", "gateway_account_id", "resource_version", "updated_at", "created_at", \
    "reference_id", "deleted"]


    @staticmethod
    def create_using_permanent_token(params, env=None, headers=None):
        return request.send('post', request.uri_path("virtual_bank_accounts","create_using_permanent_token"), params, env, headers)

    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("virtual_bank_accounts"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("virtual_bank_accounts",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("virtual_bank_accounts"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("virtual_bank_accounts",id,"delete"), None, env, headers)

    @staticmethod
    def delete_local(id, env=None, headers=None):
        return request.send('post', request.uri_path("virtual_bank_accounts",id,"delete_local"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Coupon(Model):
    class ItemConstraint(Model):
      fields = ["item_type", "constraint", "item_price_ids"]
      pass
    class ItemConstraintCriteria(Model):
      fields = ["item_type", "currencies", "item_family_ids", "item_price_periods"]
      pass

    fields = ["id", "name", "invoice_name", "discount_type", "discount_percentage", "discount_amount", \
    "discount_quantity", "currency_code", "duration_type", "duration_month", "valid_till", "max_redemptions", \
    "status", "apply_discount_on", "apply_on", "plan_constraint", "addon_constraint", "created_at", \
    "archived_at", "resource_version", "updated_at", "included_in_mrr", "plan_ids", "addon_ids", \
    "item_constraints", "item_constraint_criteria", "redemptions", "invoice_notes", "meta_data"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("coupons"), params, env, headers)

    @staticmethod
    def create_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("coupons","create_for_items"), params, env, headers)

    @staticmethod
    def update_for_items(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("coupons",id,"update_for_items"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("coupons"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("coupons",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("coupons",id), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("coupons",id,"delete"), None, env, headers)

    @staticmethod
    def copy(params, env=None, headers=None):
        return request.send('post', request.uri_path("coupons","copy"), params, env, headers)

    @staticmethod
    def unarchive(id, env=None, headers=None):
        return request.send('post', request.uri_path("coupons",id,"unarchive"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Address(Model):

    fields = ["label", "first_name", "last_name", "email", "company", "phone", "addr", "extended_addr", \
    "extended_addr2", "city", "state_code", "state", "country", "zip", "validation_status", "subscription_id"]


    @staticmethod
    def retrieve(params, env=None, headers=None):
        return request.send('get', request.uri_path("addresses"), params, env, headers)

    @staticmethod
    def update(params, env=None, headers=None):
        return request.send('post', request.uri_path("addresses"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class UnbilledCharge(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "quantity_used", "unit_amount", "starting_unit_in_decimal", "ending_unit_in_decimal", "quantity_used_in_decimal", "unit_amount_in_decimal"]
      pass

    fields = ["id", "customer_id", "subscription_id", "date_from", "date_to", "unit_amount", \
    "pricing_model", "quantity", "amount", "currency_code", "discount_amount", "description", "entity_type", \
    "entity_id", "is_voided", "voided_at", "unit_amount_in_decimal", "quantity_in_decimal", "amount_in_decimal", \
    "tiers", "deleted"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("unbilled_charges"), params, env, headers)

    @staticmethod
    def invoice_unbilled_charges(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("unbilled_charges","invoice_unbilled_charges"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("unbilled_charges",id,"delete"), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("unbilled_charges"), params, env, headers)

    @staticmethod
    def invoice_now_estimate(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("unbilled_charges","invoice_now_estimate"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class PortalSession(Model):
    class LinkedCustomer(Model):
      fields = ["customer_id", "email", "has_billing_address", "has_payment_method", "has_active_subscription"]
      pass

    fields = ["id", "token", "access_url", "redirect_url", "status", "created_at", "expires_at", \
    "customer_id", "login_at", "logout_at", "login_ipaddress", "logout_ipaddress", "linked_customers"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("portal_sessions"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("portal_sessions",id), None, env, headers)

    @staticmethod
    def logout(id, env=None, headers=None):
        return request.send('post', request.uri_path("portal_sessions",id,"logout"), None, env, headers)

    @staticmethod
    def activate(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("portal_sessions",id,"activate"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ContractTerm(Model):

    fields = ["id", "status", "contract_start", "contract_end", "billing_cycle", "action_at_term_end", \
    "total_contract_value", "cancellation_cutoff_period", "created_at", "subscription_id", "remaining_billing_cycles"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class CouponCode(Model):

    fields = ["code", "status", "coupon_id", "coupon_set_id", "coupon_set_name"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_codes"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("coupon_codes",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("coupon_codes"), params, env, headers)

    @staticmethod
    def archive(id, env=None, headers=None):
        return request.send('post', request.uri_path("coupon_codes",id,"archive"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Contact(Model):

    fields = ["id", "first_name", "last_name", "email", "phone", "label", "enabled", "send_account_email", \
    "send_billing_email"]




from chargebee.api_error import APIError,PaymentError,InvalidRequestError,OperationFailedError
from chargebee.models import *
from chargebee.main import ChargeBee


def configure(api_key, site):
    ChargeBee.configure({
        'api_key': api_key,
        'site': site,
    })



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Plan(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "price", "starting_unit_in_decimal", "ending_unit_in_decimal", "price_in_decimal"]
      pass
    class ApplicableAddon(Model):
      fields = ["id"]
      pass
    class AttachedAddon(Model):
      fields = ["id", "quantity", "billing_cycles", "type", "quantity_in_decimal"]
      pass
    class EventBasedAddon(Model):
      fields = ["id", "quantity", "on_event", "charge_once", "quantity_in_decimal"]
      pass

    fields = ["id", "name", "invoice_name", "description", "price", "currency_code", "period", \
    "period_unit", "trial_period", "trial_period_unit", "pricing_model", "charge_model", "free_quantity", \
    "setup_cost", "downgrade_penalty", "status", "archived_at", "billing_cycles", "redirect_url", \
    "enabled_in_hosted_pages", "enabled_in_portal", "addon_applicability", "tax_code", "taxjar_product_code", \
    "avalara_sale_type", "avalara_transaction_type", "avalara_service_type", "sku", "accounting_code", \
    "accounting_category1", "accounting_category2", "is_shippable", "shipping_frequency_period", \
    "shipping_frequency_period_unit", "resource_version", "updated_at", "giftable", "claim_url", \
    "free_quantity_in_decimal", "price_in_decimal", "invoice_notes", "taxable", "tax_profile_id", \
    "meta_data", "tiers", "applicable_addons", "attached_addons", "event_based_addons", "show_description_in_invoices", \
    "show_description_in_quotes"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("plans"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("plans",id), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("plans"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("plans",id), None, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("plans",id,"delete"), None, env, headers)

    @staticmethod
    def copy(params, env=None, headers=None):
        return request.send('post', request.uri_path("plans","copy"), params, env, headers)

    @staticmethod
    def unarchive(id, env=None, headers=None):
        return request.send('post', request.uri_path("plans",id,"unarchive"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ItemPrice(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "price"]
      pass
    class TaxDetail(Model):
      fields = ["tax_profile_id", "avalara_sale_type", "avalara_transaction_type", "avalara_service_type", "avalara_tax_code", "taxjar_product_code"]
      pass
    class AccountingDetail(Model):
      fields = ["sku", "accounting_code", "accounting_category1", "accounting_category2"]
      pass

    fields = ["id", "name", "item_family_id", "item_id", "description", "status", "external_name", \
    "pricing_model", "price", "period", "currency_code", "period_unit", "trial_period", "trial_period_unit", \
    "shipping_period", "shipping_period_unit", "billing_cycles", "free_quantity", "free_quantity_in_decimal", \
    "price_in_decimal", "resource_version", "updated_at", "created_at", "invoice_notes", "tiers", \
    "is_taxable", "tax_detail", "accounting_detail", "metadata", "item_type", "archivable", "parent_item_id"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("item_prices",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices",id), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("item_prices"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices",id,"delete"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class QuotedSubscription(Model):
    class Addon(Model):
      fields = ["id", "quantity", "unit_price", "amount", "trial_end", "remaining_billing_cycles", "quantity_in_decimal", "unit_price_in_decimal", "amount_in_decimal"]
      pass
    class EventBasedAddon(Model):
      fields = ["id", "quantity", "unit_price", "service_period_in_days", "on_event", "charge_once", "quantity_in_decimal", "unit_price_in_decimal"]
      pass
    class Coupon(Model):
      fields = ["coupon_id", "apply_till", "applied_count", "coupon_code"]
      pass
    class SubscriptionItem(Model):
      fields = ["item_price_id", "item_type", "quantity", "unit_price", "amount", "free_quantity", "trial_end", "billing_cycles", "service_period_days", "charge_on_event", "charge_once", "charge_on_option"]
      pass
    class ItemTier(Model):
      fields = ["item_price_id", "starting_unit", "ending_unit", "price"]
      pass

    fields = ["id", "plan_id", "plan_quantity", "plan_unit_price", "setup_fee", "billing_period", \
    "billing_period_unit", "start_date", "trial_end", "remaining_billing_cycles", "po_number", "auto_collection", \
    "addons", "event_based_addons", "coupons", "subscription_items", "item_tiers"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Card(Model):

    fields = ["payment_source_id", "status", "gateway", "gateway_account_id", "ref_tx_id", \
    "first_name", "last_name", "iin", "last4", "card_type", "funding_type", "expiry_month", "expiry_year", \
    "issuing_country", "billing_addr1", "billing_addr2", "billing_city", "billing_state_code", "billing_state", \
    "billing_country", "billing_zip", "created_at", "resource_version", "updated_at", "ip_address", \
    "powered_by", "customer_id", "masked_number"]


    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("cards",id), None, env, headers)

    @staticmethod
    def update_card_for_customer(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"credit_card"), params, env, headers)

    @staticmethod
    def switch_gateway_for_customer(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"switch_gateway"), params, env, headers)

    @staticmethod
    def copy_card_for_customer(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"copy_card"), params, env, headers)

    @staticmethod
    def delete_card_for_customer(id, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"delete_card"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class CreditNote(Model):
    class LineItem(Model):
      fields = ["id", "subscription_id", "date_from", "date_to", "unit_amount", "quantity", "amount", "pricing_model", "is_taxed", "tax_amount", "tax_rate", "unit_amount_in_decimal", "quantity_in_decimal", "amount_in_decimal", "discount_amount", "item_level_discount_amount", "description", "entity_description", "entity_type", "tax_exempt_reason", "entity_id", "customer_id"]
      pass
    class Discount(Model):
      fields = ["amount", "description", "entity_type", "entity_id"]
      pass
    class LineItemDiscount(Model):
      fields = ["line_item_id", "discount_type", "coupon_id", "discount_amount"]
      pass
    class LineItemTier(Model):
      fields = ["line_item_id", "starting_unit", "ending_unit", "quantity_used", "unit_amount", "starting_unit_in_decimal", "ending_unit_in_decimal", "quantity_used_in_decimal", "unit_amount_in_decimal"]
      pass
    class Tax(Model):
      fields = ["name", "amount", "description"]
      pass
    class LineItemTax(Model):
      fields = ["line_item_id", "tax_name", "tax_rate", "is_partial_tax_applied", "is_non_compliance_tax", "taxable_amount", "tax_amount", "tax_juris_type", "tax_juris_name", "tax_juris_code", "tax_amount_in_local_currency", "local_currency_code"]
      pass
    class LinkedRefund(Model):
      fields = ["txn_id", "applied_amount", "applied_at", "txn_status", "txn_date", "txn_amount", "refund_reason_code"]
      pass
    class Allocation(Model):
      fields = ["invoice_id", "allocated_amount", "allocated_at", "invoice_date", "invoice_status"]
      pass

    fields = ["id", "customer_id", "subscription_id", "reference_invoice_id", "type", "reason_code", \
    "status", "vat_number", "date", "price_type", "currency_code", "total", "amount_allocated", \
    "amount_refunded", "amount_available", "refunded_at", "voided_at", "resource_version", "updated_at", \
    "sub_total", "sub_total_in_local_currency", "total_in_local_currency", "local_currency_code", \
    "round_off_amount", "fractional_correction", "line_items", "discounts", "line_item_discounts", \
    "line_item_tiers", "taxes", "line_item_taxes", "linked_refunds", "allocations", "deleted", "create_reason_code"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("credit_notes",id), None, env, headers)

    @staticmethod
    def pdf(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes",id,"pdf"), params, env, headers)

    @staticmethod
    def refund(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes",id,"refund"), params, env, headers)

    @staticmethod
    def record_refund(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes",id,"record_refund"), params, env, headers)

    @staticmethod
    def void_credit_note(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes",id,"void"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("credit_notes"), params, env, headers)

    @staticmethod
    def credit_notes_for_customer(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"credit_notes"), params, env, headers)

    @staticmethod
    def delete(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("credit_notes",id,"delete"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Token(Model):

    fields = ["id", "gateway", "gateway_account_id", "payment_method_type", "status", "id_at_vault", \
    "vault", "ip_address", "created_at", "expired_at"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class SubscriptionEstimate(Model):
    class ShippingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class ContractTerm(Model):
      fields = ["id", "status", "contract_start", "contract_end", "billing_cycle", "action_at_term_end", "total_contract_value", "cancellation_cutoff_period", "created_at", "subscription_id", "remaining_billing_cycles"]
      pass

    fields = ["id", "currency_code", "status", "next_billing_at", "pause_date", "resume_date", \
    "shipping_address", "contract_term"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Transaction(Model):
    class LinkedInvoice(Model):
      fields = ["invoice_id", "applied_amount", "applied_at", "invoice_date", "invoice_total", "invoice_status"]
      pass
    class LinkedCreditNote(Model):
      fields = ["cn_id", "applied_amount", "applied_at", "cn_reason_code", "cn_create_reason_code", "cn_date", "cn_total", "cn_status", "cn_reference_invoice_id"]
      pass
    class LinkedRefund(Model):
      fields = ["txn_id", "txn_status", "txn_date", "txn_amount"]
      pass
    class LinkedPayment(Model):
      fields = ["id", "status", "amount", "date"]
      pass

    fields = ["id", "customer_id", "subscription_id", "gateway_account_id", "payment_source_id", \
    "payment_method", "reference_number", "gateway", "type", "date", "settled_at", "currency_code", \
    "amount", "id_at_gateway", "status", "fraud_flag", "initiator_type", "three_d_secure", "authorization_reason", \
    "error_code", "error_text", "voided_at", "resource_version", "updated_at", "fraud_reason", "amount_unused", \
    "masked_card_number", "reference_transaction_id", "refunded_txn_id", "reference_authorization_id", \
    "amount_capturable", "reversal_transaction_id", "linked_invoices", "linked_credit_notes", "linked_refunds", \
    "linked_payments", "deleted"]


    @staticmethod
    def create_authorization(params, env=None, headers=None):
        return request.send('post', request.uri_path("transactions","create_authorization"), params, env, headers)

    @staticmethod
    def void_transaction(id, env=None, headers=None):
        return request.send('post', request.uri_path("transactions",id,"void"), None, env, headers)

    @staticmethod
    def record_refund(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("transactions",id,"record_refund"), params, env, headers)

    @staticmethod
    def refund(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("transactions",id,"refund"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("transactions"), params, env, headers)

    @staticmethod
    def transactions_for_customer(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("customers",id,"transactions"), params, env, headers)

    @staticmethod
    def transactions_for_subscription(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"transactions"), params, env, headers)

    @staticmethod
    def payments_for_invoice(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("invoices",id,"payments"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("transactions",id), None, env, headers)

    @staticmethod
    def delete_offline_transaction(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("transactions",id,"delete_offline_transaction"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class PaymentSource(Model):
    class Card(Model):
      fields = ["first_name", "last_name", "iin", "last4", "brand", "funding_type", "expiry_month", "expiry_year", "billing_addr1", "billing_addr2", "billing_city", "billing_state_code", "billing_state", "billing_country", "billing_zip", "masked_number"]
      pass
    class BankAccount(Model):
      fields = ["last4", "name_on_account", "bank_name", "mandate_id", "account_type", "echeck_type", "account_holder_type"]
      pass
    class AmazonPayment(Model):
      fields = ["email", "agreement_id"]
      pass
    class Paypal(Model):
      fields = ["email", "agreement_id"]
      pass

    fields = ["id", "resource_version", "updated_at", "created_at", "customer_id", "type", \
    "reference_id", "status", "gateway", "gateway_account_id", "ip_address", "issuing_country", \
    "card", "bank_account", "amazon_payment", "paypal", "deleted"]


    @staticmethod
    def create_using_temp_token(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_using_temp_token"), params, env, headers)

    @staticmethod
    def create_using_permanent_token(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_using_permanent_token"), params, env, headers)

    @staticmethod
    def create_using_token(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_using_token"), params, env, headers)

    @staticmethod
    def create_using_payment_intent(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_using_payment_intent"), params, env, headers)

    @staticmethod
    def create_card(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_card"), params, env, headers)

    @staticmethod
    def create_bank_account(params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources","create_bank_account"), params, env, headers)

    @staticmethod
    def update_card(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"update_card"), params, env, headers)

    @staticmethod
    def verify_bank_account(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"verify_bank_account"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("payment_sources",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("payment_sources"), params, env, headers)

    @staticmethod
    def switch_gateway_account(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"switch_gateway_account"), params, env, headers)

    @staticmethod
    def export_payment_source(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"export_payment_source"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"delete"), None, env, headers)

    @staticmethod
    def delete_local(id, env=None, headers=None):
        return request.send('post', request.uri_path("payment_sources",id,"delete_local"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Gift(Model):
    class Gifter(Model):
      fields = ["customer_id", "invoice_id", "signature", "note"]
      pass
    class GiftReceiver(Model):
      fields = ["customer_id", "subscription_id", "first_name", "last_name", "email"]
      pass
    class GiftTimeline(Model):
      fields = ["status", "occurred_at"]
      pass

    fields = ["id", "status", "scheduled_at", "auto_claim", "no_expiry", "claim_expiry_date", \
    "resource_version", "updated_at", "gifter", "gift_receiver", "gift_timelines"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("gifts"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("gifts",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("gifts"), params, env, headers)

    @staticmethod
    def claim(id, env=None, headers=None):
        return request.send('post', request.uri_path("gifts",id,"claim"), None, env, headers)

    @staticmethod
    def cancel(id, env=None, headers=None):
        return request.send('post', request.uri_path("gifts",id,"cancel"), None, env, headers)

    @staticmethod
    def update_gift(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("gifts",id,"update_gift"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class SiteMigrationDetail(Model):

    fields = ["entity_id", "other_site_name", "entity_id_at_other_site", "migrated_at", "entity_type", \
    "status"]


    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("site_migration_details"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Quote(Model):
    class LineItem(Model):
      fields = ["id", "subscription_id", "date_from", "date_to", "unit_amount", "quantity", "amount", "pricing_model", "is_taxed", "tax_amount", "tax_rate", "unit_amount_in_decimal", "quantity_in_decimal", "amount_in_decimal", "discount_amount", "item_level_discount_amount", "description", "entity_description", "entity_type", "tax_exempt_reason", "entity_id", "customer_id"]
      pass
    class Discount(Model):
      fields = ["amount", "description", "entity_type", "entity_id"]
      pass
    class LineItemDiscount(Model):
      fields = ["line_item_id", "discount_type", "coupon_id", "discount_amount"]
      pass
    class Tax(Model):
      fields = ["name", "amount", "description"]
      pass
    class LineItemTax(Model):
      fields = ["line_item_id", "tax_name", "tax_rate", "is_partial_tax_applied", "is_non_compliance_tax", "taxable_amount", "tax_amount", "tax_juris_type", "tax_juris_name", "tax_juris_code", "tax_amount_in_local_currency", "local_currency_code"]
      pass
    class ShippingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class BillingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass

    fields = ["id", "name", "po_number", "customer_id", "subscription_id", "invoice_id", "status", \
    "operation_type", "vat_number", "price_type", "valid_till", "date", "total_payable", "charge_on_acceptance", \
    "sub_total", "total", "credits_applied", "amount_paid", "amount_due", "version", "resource_version", \
    "updated_at", "line_items", "discounts", "line_item_discounts", "taxes", "line_item_taxes", \
    "currency_code", "notes", "shipping_address", "billing_address", "contract_term_start", "contract_term_end", \
    "contract_term_termination_fee"]


    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("quotes",id), None, env, headers)

    @staticmethod
    def create_sub_for_customer_quote(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"create_subscription_quote"), params, env, headers)

    @staticmethod
    def edit_create_sub_for_customer_quote(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"edit_create_subscription_quote"), params, env, headers)

    @staticmethod
    def create_sub_items_for_customer_quote(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("customers",id,"create_subscription_quote_for_items"), params, env, headers)

    @staticmethod
    def update_subscription_quote(params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes","update_subscription_quote"), params, env, headers)

    @staticmethod
    def edit_update_subscription_quote(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"edit_update_subscription_quote"), params, env, headers)

    @staticmethod
    def update_subscription_quote_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes","update_subscription_quote_for_items"), params, env, headers)

    @staticmethod
    def create_for_onetime_charges(params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes","create_for_onetime_charges"), params, env, headers)

    @staticmethod
    def create_for_charge_items_and_charges(params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes","create_for_charge_items_and_charges"), params, env, headers)

    @staticmethod
    def edit_one_time_quote(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"edit_one_time_quote"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("quotes"), params, env, headers)

    @staticmethod
    def quote_line_groups_for_quote(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("quotes",id,"quote_line_groups"), params, env, headers)

    @staticmethod
    def convert(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"convert"), params, env, headers)

    @staticmethod
    def update_status(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"update_status"), params, env, headers)

    @staticmethod
    def extend_expiry_date(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"extend_expiry_date"), params, env, headers)

    @staticmethod
    def delete(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"delete"), params, env, headers)

    @staticmethod
    def pdf(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("quotes",id,"pdf"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class PromotionalCredit(Model):

    fields = ["id", "customer_id", "type", "amount_in_decimal", "amount", "currency_code", \
    "description", "credit_type", "reference", "closing_balance", "done_by", "created_at"]


    @staticmethod
    def add(params, env=None, headers=None):
        return request.send('post', request.uri_path("promotional_credits","add"), params, env, headers)

    @staticmethod
    def deduct(params, env=None, headers=None):
        return request.send('post', request.uri_path("promotional_credits","deduct"), params, env, headers)

    @staticmethod
    def set(params, env=None, headers=None):
        return request.send('post', request.uri_path("promotional_credits","set"), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("promotional_credits"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("promotional_credits",id), None, env, headers)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys
import os

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'chargebee'))
import version

requires = []
try:
    import json
except ImportError:
    try:
        import simplejson
    except ImportError:
        requires.append('simplejson')  # For Python==2.5

setup(
    name='chargebee',
    version=version.VERSION,
    author='ChargeBee',
    author_email='support@chargebee.com',
    url='https://apidocs.chargebee.com/docs/api?lang=python',
    description='Python wrapper for the ChargeBee Subscription Billing API',
    packages=find_packages(exclude=["tests"]),
    package_data={'chargebee': ['ssl/*.crt']},
    install_requires=requires,
    test_suite='tests',
)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ResourceMigration(Model):

    fields = ["from_site", "entity_type", "entity_id", "status", "errors", "created_at", "updated_at"]


    @staticmethod
    def retrieve_latest(params, env=None, headers=None):
        return request.send('get', request.uri_path("resource_migrations","retrieve_latest"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Order(Model):
    class OrderLineItem(Model):
      fields = ["id", "invoice_id", "invoice_line_item_id", "unit_price", "description", "amount", "fulfillment_quantity", "fulfillment_amount", "tax_amount", "amount_paid", "amount_adjusted", "refundable_credits_issued", "refundable_credits", "is_shippable", "sku", "status", "entity_type", "item_level_discount_amount", "discount_amount", "entity_id"]
      pass
    class ShippingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class BillingAddress(Model):
      fields = ["first_name", "last_name", "email", "company", "phone", "line1", "line2", "line3", "city", "state_code", "state", "country", "zip", "validation_status"]
      pass
    class LineItemTax(Model):
      fields = ["line_item_id", "tax_name", "tax_rate", "is_partial_tax_applied", "is_non_compliance_tax", "taxable_amount", "tax_amount", "tax_juris_type", "tax_juris_name", "tax_juris_code", "tax_amount_in_local_currency", "local_currency_code"]
      pass
    class LineItemDiscount(Model):
      fields = ["line_item_id", "discount_type", "coupon_id", "discount_amount"]
      pass
    class LinkedCreditNote(Model):
      fields = ["amount", "type", "id", "status", "amount_adjusted", "amount_refunded"]
      pass

    fields = ["id", "document_number", "invoice_id", "subscription_id", "customer_id", "status", \
    "cancellation_reason", "payment_status", "order_type", "price_type", "reference_id", "fulfillment_status", \
    "order_date", "shipping_date", "note", "tracking_id", "batch_id", "created_by", "shipment_carrier", \
    "invoice_round_off_amount", "tax", "amount_paid", "amount_adjusted", "refundable_credits_issued", \
    "refundable_credits", "rounding_adjustement", "paid_on", "shipping_cut_off_date", "created_at", \
    "status_update_at", "delivered_at", "shipped_at", "resource_version", "updated_at", "cancelled_at", \
    "order_line_items", "shipping_address", "billing_address", "discount", "sub_total", "total", \
    "line_item_taxes", "line_item_discounts", "linked_credit_notes", "deleted", "currency_code", \
    "is_gifted", "gift_note", "gift_id"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("orders"), params, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id), params, env, headers)

    @staticmethod
    def import_order(params, env=None, headers=None):
        return request.send('post', request.uri_path("orders","import_order"), params, env, headers)

    @staticmethod
    def assign_order_number(id, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id,"assign_order_number"), None, env, headers)

    @staticmethod
    def cancel(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id,"cancel"), params, env, headers)

    @staticmethod
    def create_refundable_credit_note(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id,"create_refundable_credit_note"), params, env, headers)

    @staticmethod
    def reopen(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id,"reopen"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("orders",id), None, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("orders",id,"delete"), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("orders"), params, env, headers)

    @staticmethod
    def orders_for_invoice(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("invoices",id,"orders"), params, env, headers)




from chargebee.main import Environment

import re
import sys
import socket

try:
    import simplejson as json
except ImportError:
    import json

py_major_v = sys.version_info[0]
py_minor_v = sys.version_info[1]

if py_major_v < 3:
    from urllib import urlencode
    from urlparse import urlparse
    from urllib2 import urlopen as _urlopen, Request
elif py_major_v >= 3:
    from urllib.parse import urlencode, urlparse
    from urllib.request import urlopen as _urlopen, Request



try:
    SSLError = None
    ssl = None
    
    if Environment.chargebee_domain is None:
        HTTPSConnection = object
    else:
        HTTPConnection = object
            
    if py_major_v < 3:
        from httplib import HTTPConnection, HTTPSConnection, HTTPException
    else:
        from http.client import HTTPConnection, HTTPSConnection, HTTPException

    import ssl
    SSLError = ssl.SSLError

except (ImportError, AttributeError):  # Platform-specific: No SSL
    pass


class VerifiedHTTPSConnection(HTTPSConnection):
    """Based on httplib.HTTPSConnection but wraps socket with SSL certification"""

    def set_cert(self, ca_certs, cert_reqs='CERT_REQUIRED'):
        ssl_req_scheme = {
            'CERT_NONE': ssl.CERT_NONE,
            'CERT_OPTIONAL': ssl.CERT_OPTIONAL,
            'CERT_REQUIRED': ssl.CERT_REQUIRED
        }

        self.cert_reqs = ssl_req_scheme.get(cert_reqs) or ssl.CERT_NONE
        self.ca_certs = ca_certs

    def connect(self):
        # Add certificate verification
        sock = socket.create_connection((self.host, self.port), self.timeout)

        # Wrap socket using verification with the root certs in
        # trusted_root_certs
        self.sock = ssl.wrap_socket(sock, cert_reqs=self.cert_reqs, ca_certs=self.ca_certs)

        if self.ca_certs:
            match_hostname(self.sock.getpeercert(), self.host)


class CertificateError(ValueError):
    pass


def _dnsname_to_pat(dn):
    pats = []
    for frag in dn.split(r'.'):
        if frag == '*':
            # When '*' is a fragment by itself, it matches a non-empty dotless
            # fragment.
            pats.append('[^.]+')
        else:
            # Otherwise, '*' matches any dotless fragment.
            frag = re.escape(frag)
            pats.append(frag.replace(r'\*', '[^.]*'))
    return re.compile(r'\A' + r'\.'.join(pats) + r'\Z', re.IGNORECASE)


def match_hostname(cert, hostname):
    """Verify that *cert* (in decoded format as returned by
    SSLSocket.getpeercert()) matches the *hostname*.  RFC 2818 rules
    are mostly followed, but IP addresses are not accepted for *hostname*.

    CertificateError is raised on failure. On success, the function
    returns nothing.
    """

    if not cert:
        raise ValueError("empty or no certificate")
    dnsnames = []
    san = cert.get('subjectAltName', ())
    for key, value in san:
        if key == 'DNS':
            if _dnsname_to_pat(value).match(hostname):
                return
            dnsnames.append(value)
    if not dnsnames:
        # The subject is only checked when there is no dNSName entry
        # in subjectAltName
        for sub in cert.get('subject', ()):
            for key, value in sub:
                # XXX according to RFC 2818, the most specific Common Name
                # must be used.
                if key == 'commonName':
                    if _dnsname_to_pat(value).match(hostname):
                        return
                    dnsnames.append(value)
    if len(dnsnames) > 1:
        raise CertificateError("hostname %r "
                               "doesn't match either of %s"
                               % (hostname, ', '.join(map(repr, dnsnames))))
    elif len(dnsnames) == 1:
        raise CertificateError("hostname %r "
                               "doesn't match %r"
                               % (hostname, dnsnames[0]))
    else:
        raise CertificateError("no appropriate commonName or "
                               "subjectAltName fields were found")



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Item(Model):
    class ApplicableItem(Model):
      fields = ["id"]
      pass

    fields = ["id", "name", "description", "status", "resource_version", "updated_at", "item_family_id", \
    "type", "is_shippable", "is_giftable", "redirect_url", "enabled_for_checkout", "enabled_in_portal", \
    "included_in_mrr", "item_applicability", "gift_claim_redirect_url", "unit", "applicable_items", \
    "metadata"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("items"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("items",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("items",id), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("items"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("items",id,"delete"), None, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Export(Model):
    class Download(Model):
      fields = ["download_url", "valid_till"]
      pass

    fields = ["id", "operation_type", "mime_type", "status", "created_at", "download"]
    def wait_for_export_completion(self):
        return wait_for_export_completion()

    def wait_for_export_completion(self, env=None, headers=None):
        import time
        count = 0
        sleep_time_millis = (10000 if env == None else env.export_sleep_millis)/1000.0

        while self.status == 'in_process':
            if count > 50:
                raise RuntimeError('Export is taking too long')
            count+=1
            time.sleep(sleep_time_millis);
            self.values = Export.retrieve(self.id, env, headers).export.values
            self.load(self.values)
        return self


    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("exports",id), None, env, headers)

    @staticmethod
    def revenue_recognition(params, env=None, headers=None):
        return request.send('post', request.uri_path("exports","revenue_recognition"), params, env, headers)

    @staticmethod
    def deferred_revenue(params, env=None, headers=None):
        return request.send('post', request.uri_path("exports","deferred_revenue"), params, env, headers)

    @staticmethod
    def plans(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","plans"), params, env, headers)

    @staticmethod
    def addons(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","addons"), params, env, headers)

    @staticmethod
    def coupons(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","coupons"), params, env, headers)

    @staticmethod
    def customers(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","customers"), params, env, headers)

    @staticmethod
    def subscriptions(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","subscriptions"), params, env, headers)

    @staticmethod
    def invoices(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","invoices"), params, env, headers)

    @staticmethod
    def credit_notes(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","credit_notes"), params, env, headers)

    @staticmethod
    def transactions(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","transactions"), params, env, headers)

    @staticmethod
    def orders(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","orders"), params, env, headers)

    @staticmethod
    def item_families(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","item_families"), params, env, headers)

    @staticmethod
    def items(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","items"), params, env, headers)

    @staticmethod
    def item_prices(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","item_prices"), params, env, headers)

    @staticmethod
    def attached_items(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","attached_items"), params, env, headers)

    @staticmethod
    def differential_prices(params=None, env=None, headers=None):
        return request.send('post', request.uri_path("exports","differential_prices"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class HostedPage(Model):

    fields = ["id", "type", "url", "state", "failure_reason", "pass_thru_content", "embed", \
    "created_at", "expires_at", "updated_at", "resource_version", "checkout_info"]

    @property
    def content(self):
        from chargebee import Content
        if 'content' in self.values:
            return Content(self.values['content'])
        return None

    @staticmethod
    def checkout_new(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","checkout_new"), params, env, headers)

    @staticmethod
    def checkout_new_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","checkout_new_for_items"), params, env, headers)

    @staticmethod
    def checkout_existing(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","checkout_existing"), params, env, headers)

    @staticmethod
    def checkout_existing_for_items(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","checkout_existing_for_items"), params, env, headers)

    @staticmethod
    def update_card(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","update_card"), params, env, headers)

    @staticmethod
    def update_payment_method(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","update_payment_method"), params, env, headers)

    @staticmethod
    def manage_payment_sources(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","manage_payment_sources"), params, env, headers)

    @staticmethod
    def collect_now(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","collect_now"), params, env, headers)

    @staticmethod
    def accept_quote(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","accept_quote"), params, env, headers)

    @staticmethod
    def extend_subscription(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","extend_subscription"), params, env, headers)

    @staticmethod
    def checkout_gift(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","checkout_gift"), params, env, headers)

    @staticmethod
    def claim_gift(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","claim_gift"), params, env, headers)

    @staticmethod
    def retrieve_agreement_pdf(params, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages","retrieve_agreement_pdf"), params, env, headers)

    @staticmethod
    def acknowledge(id, env=None, headers=None):
        return request.send('post', request.uri_path("hosted_pages",id,"acknowledge"), None, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("hosted_pages",id), None, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("hosted_pages"), params, env, headers)



from chargebee.models.addon import Addon
from chargebee.models.address import Address
from chargebee.models.card import Card
from chargebee.models.coupon import Coupon
from chargebee.models.coupon_code import CouponCode
from chargebee.models.coupon_set import CouponSet
from chargebee.models.customer import Customer
from chargebee.models.event import Event
from chargebee.models.hosted_page import HostedPage
from chargebee.models.invoice import Invoice
from chargebee.models.credit_note import CreditNote
from chargebee.models.order import Order
from chargebee.models.estimate import Estimate
from chargebee.models.subscription_estimate import SubscriptionEstimate
from chargebee.models.invoice_estimate import InvoiceEstimate
from chargebee.models.credit_note_estimate import CreditNoteEstimate
from chargebee.models.plan import Plan
from chargebee.models.subscription import Subscription
from chargebee.models.transaction import Transaction
from chargebee.models.comment import Comment
from chargebee.models.portal_session import PortalSession
from chargebee.models.download import Download
from chargebee.models.third_party_payment_method import ThirdPartyPaymentMethod
from chargebee.models.site_migration_detail import SiteMigrationDetail
from chargebee.models.resource_migration import ResourceMigration
from chargebee.models.payment_source import PaymentSource
from chargebee.models.unbilled_charge import UnbilledCharge
from chargebee.models.time_machine import TimeMachine
from chargebee.models.promotional_credit import PromotionalCredit
from chargebee.models.virtual_bank_account import VirtualBankAccount
from chargebee.models.contact import Contact
from chargebee.models.export import Export
from chargebee.models.gift import Gift
from chargebee.models.quote import Quote
from chargebee.models.content import Content
from chargebee.models.hierarchy import Hierarchy
from chargebee.models.payment_intent import PaymentIntent